import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;

/**
 * Testklasse zu den berlegungen zur
 * Optimierung der Berechnung von Zeitrumen,
 * Terminen und Reihen in einer Kalender-Datenstruktur
 * (Datums und/oder Uhrzeit) auf Basis
 * der Expression Engine aus dem JSP-Workflow-Framework
 * Control-and-Command.
 *
 * @author Heiner Kcker
 */
public class TimeLineOperations {

    /**
     * Methode zur Ausgabe der Zeitrume und Termine
     * in ASCII-Art.
     *
     * @author hkuecker
     */
    public static String buildOszillogramm(//--
            final TimeEvent[] pTimeEventArr)
    {
        check(pTimeEventArr);
        // TODO vielleicht sortieren
        final int maxTime = pTimeEventArr[ pTimeEventArr.length - 1].time;

        char[] topCharArr = new char[ maxTime + 1 ];
        char[] betCharArr = new char[ maxTime + 1 ];
        char[] botCharArr = new char[ maxTime + 1 ];

        Arrays.fill(topCharArr, ' ');
        Arrays.fill(betCharArr, ' ');
        Arrays.fill(botCharArr, ' ');

        int chrPos = 0;
        for (int i = 0; i < pTimeEventArr.length; i++)
        {
            final TimeEvent timeEvent = pTimeEventArr[i];
            while (chrPos < timeEvent.time ){
                paintPre( //--
                        topCharArr, //--
                        betCharArr, //--
                        botCharArr, //--
                        chrPos, //--
                        timeEvent.type);
                chrPos++;
            }
            if (chrPos < maxTime)
            {
                paintBetween( //--
                        topCharArr, //--
                        betCharArr, //--
                        botCharArr, //--
                        chrPos, //--
                        timeEvent.type);
                chrPos++;
            }
        }
        return "1 " +new String(topCharArr) + '\n' //--
             + "  " + new String(betCharArr) + '\n' //--
             + "0 " + new String(botCharArr) + '\n' ;
    }

	  /**
     * Zeichnen des Zustandes vor dem Stichzeitpunkt
     * fr den brgebenen Zeitpunkt-Type.
     *
     * @param pTopCharArr
     * @param pBetCharArr
     * @param pBotCharArr
     * @param pType
     */
    private static void paintPre(//--
            final char[] pTopCharArr, //--
            final char[] pBetCharArr, //--
            final char[] pBotCharArr, //--
            final int pChrPos, //--
            final TimeEventType pTimeEventType) {
        if (pTimeEventType.getPreStat())
        {
            pTopCharArr[pChrPos] = '-';
            return;
        }
        pBotCharArr[pChrPos] = '-';
    }

    /**
     * Zeichnen des Zustandes whrend des
     * Stichzeitpunkts fr den brgebenen
     * Zeitpunkt-Type.
     *
     * @param pTopCharArr
     * @param pBetCharArr
     * @param pBotCharArr
     * @param pType
     */
    private static void paintBetween(//--
            final char[] pTopCharArr, //--
            final char[] pBetCharArr, //--
            final char[] pBotCharArr, //--
            final int pChrPos, //--
            final TimeEventType pTimeEventType) {
        if (pTimeEventType.equals(TimeEventType.START_0) //--
                || pTimeEventType.equals(TimeEventType.END_0))
        {
            pBotCharArr[pChrPos] = '-';
            return;
        }
        if (pTimeEventType.equals(TimeEventType.START_1) //--
                || pTimeEventType.equals(TimeEventType.END_1))
        {
            pTopCharArr[pChrPos] = '-';
            return;
        }
        if (pTimeEventType.equals(TimeEventType.FROM_0_TO_1) //--
                || pTimeEventType.equals(TimeEventType.FROM_1_TO_0))
        {
            pTopCharArr[pChrPos] = '+';
            pBetCharArr[pChrPos] = '|';
            pBotCharArr[pChrPos] = '+';
            return;
        }
        if (pTimeEventType.equals(TimeEventType.MOMENT_1))
        {
            pTopCharArr[pChrPos] = '|';
            pBetCharArr[pChrPos] = '|';
            pBotCharArr[pChrPos] = '+';
            return;
        }
        if (pTimeEventType.equals(TimeEventType.MOMENT_0))
        {
            pTopCharArr[pChrPos] = '+';
            pBetCharArr[pChrPos] = '|';
            pBotCharArr[pChrPos] = '|';
            return;
        }
    }

    /**
     * Prfen der Integritt einer Timeline.
     * @param pTimeEventArr
     */
    private static void check(final TimeEvent[] pTimeEventArr)
    {
    	//check mindestens ein Start- und ein End-Zeitpunkt
        if (pTimeEventArr.length < 2){
         throw new RuntimeException("min length two elements expected");
        }
        //check Start-Zeitpunkt
        if (pTimeEventArr[0].type != TimeEventType.START_0
                && pTimeEventArr[0].type != TimeEventType.START_1 )
        {
            throw new RuntimeException("start element expected");
        }
        //check End-Zeitpunkt
        if (pTimeEventArr[ pTimeEventArr.length - 1 ].type != TimeEventType.END_0
                && pTimeEventArr[ pTimeEventArr.length - 1 ].type != TimeEventType.END_1 )
        {
            throw new RuntimeException("end element expected");
        }
        //check next.time >= current.time
        //check kein Start und Ende an inneren Positionen
        //check keine redundanten Ereignisse 1 -> 1 or 0 -> 0
    }




    /**
     * Enum-Klasse fr die Arten der Ereignisse.
     * <pre>
     * |Zeitpunkt |  Ereigniss  |      Bemerkung          |              Ausdruck              |
     * +----------+-------------+-------------------------+------------------------------------+
     * | Start    |      0      | Initialer Zustand 0     | date > oder >= xx.xx.xxDateLiteral |
     * | Start    |      1      | Initialer Zustand 1     | date < oder <= xx.xx.xxDateLiteral |
     * |  Tx      |    0 -> 1   | Beginn eines Zeitraumes | date > oder >= xx.xx.xxDateLiteral |
     * |  Tx      |    1 -> 0   | Ende   eines Zeitraumes | date < oder <= xx.xx.xxDateLiteral |
     * |  Tx      | 0 -> 1 -> 0 | Termin / Ereigniss      | date == xx.xx.xxDateLiteral        |
     * |  Tx      | 1 -> 0 -> 1 | Termin / Ereigniss      | date != xx.xx.xxDateLiteral        |
     * | Ende     |      0      | End-Zustand 0           | date < oder <= xx.xx.xxDateLiteral |
     * | Ende     |      1      | End-Zustand 1           | date > oder >= xx.xx.xxDateLiteral |
     * </pre>
     *
     * @author hkuecker
     */
    public static class TimeEventType
    {
        public static final TimeEventType START_0     = new TimeEventType("START_0",false,false,false);
        public static final TimeEventType START_1     = new TimeEventType("START_1",true, true, true);
        public static final TimeEventType FROM_0_TO_1 = new TimeEventType("FROM_0_TO_1",false,true,true);
        public static final TimeEventType FROM_1_TO_0 = new TimeEventType("FROM_1_TO_0",true,false,false);
        public static final TimeEventType MOMENT_1    = new TimeEventType("MOMENT_1",false,true,false);
        public static final TimeEventType MOMENT_0    = new TimeEventType("MOMENT_0",true,false,true);
        public static final TimeEventType END_0       = new TimeEventType("END_0",false,false,false);
        public static final TimeEventType END_1       = new TimeEventType("END_1",true, true, true);

        /** Name des Ereigniss-Types */
        private String name;

        /** Zustand vor dem Ereigniss */
        private boolean preStat;

        /** Zustand whrend des Ereignisses */
        private boolean momentStat;

        /** Zustand nach dem Ereigniss */
        private boolean postStat;

        private TimeEventType( //--
                final String pName, //--
                final boolean pPreStat, //--
                final boolean pMomentStat, //--
                final boolean pPostStat)
        {
            this.name = pName;
            preStat = pPreStat;
            momentStat = pMomentStat;
            postStat = pPostStat;
        }

        /**
         * toString.
         */
        public String toString()
        {
            return name;
        }
        /**
         * @return Returns the momentStat.
         */
        public boolean getMomentStat()
        {
            return this.momentStat;
        }
        /**
         * @return Returns the postStat.
         */
        public boolean getPostStat()
        {
            return this.postStat;
        }
        /**
         * @return Returns the preStat.
         */
        public boolean getPreStat()
        {
            return this.preStat;
        }
    }

    /**
     * Diese Klasse reprsentiert einen
     * Termin oder Anfang bzw. Ende eines Zeitraumes.
     *
     * @author hkuecker
     */
    public static class TimeEvent
    implements Comparable
    {
        /**
         * der Zeitpunkt.
         * spter hier Date oder Calendar einsetzen.
         */
        public int time;

        /**
         * der Type des Ereignisses
         */
        public TimeEventType type;

        /**
         * Konstruktor,
         * @param pTime
         * @param pType
         */
        public TimeEvent( //--
                final int pTime, //--
                final TimeEventType pType )
        {
            this.time = pTime;
            this.type = pType;
        }

        /**
         * @see java.lang.Comparable#compareTo(java.lang.Object)
         */
        public int compareTo(Object pO)
        {
            return time - ( (TimeEvent) pO ).time;
        }

        /**
         * @see java.lang.Object#equals(java.lang.Object)
         */
        public boolean equals(Object pObj)
        {
            if (pObj == null){
                return false;
            }
            if (pObj == this){
                return true;
            }
            if (!(pObj instanceof TimeEvent)){
                return false;
            }
            TimeEvent other = (TimeEvent) pObj;
            if (other.time == this.time //--
                    && other.type == this.type){
                return true;
            }
            return false;
        }
        /**
         * @see java.lang.Object#hashCode()
         */
        public int hashCode()
        {
            return time;
        }
        /**
         * @see java.lang.Object#toString()
         */
        public String toString()
        {
            return this.getClass().getName() //--
            + "[time: " + time //--
            + ", type: " + type //--
            + "]";
        }

        /**
         * Hilfsmethode toArray.
         * @param pCollection
         * @return Array
         */
        public static TimeEvent[] toArray(final Collection pCollection)
        {
            return (TimeEvent[]) pCollection.toArray(new TimeEvent[ pCollection.size() ]);
        }
    }

    /**
     * Klasse zum Halten eines {@link TimeEvent} fr
     * die die logischeVerknpfung (UND, ODER, ENTWEDER-ODER)
     * von Termin oder Zeitrumen mit Erhaltung der Information,
     * ob das gehaltene {@link TimeEvent} im linken oder rechten
     * Operand enthalten war.
     *
     * @author hkuecker
     */
    public static class TimeEventHolder
    implements Comparable
    {

        /**
         * gehaltenes TimeEvent.
         */
        public final TimeEvent timeEvent;

        /**
         * ob linker Operand (true)
         * oder rechter Operand (false):
         */
        public final boolean isLeftTimeEvent;

        /**
         * Konstruktor.
         * @param pTimeEvent zu haltendes TimeEvent
         * @param pIsLeftTimeEvent linker Operand 'true', rechter Operand 'false'
         */
        public TimeEventHolder( //--
                final TimeEvent pTimeEvent, //--
                final boolean pIsLeftTimeEvent)
        {
            this.timeEvent = pTimeEvent;
            this.isLeftTimeEvent = pIsLeftTimeEvent;
        }

        /**
         * compareTo
         * @param pO
         * @return
         */
        public int compareTo(Object pO)
        {
            return this.timeEvent.compareTo( ( (TimeEventHolder) pO ).timeEvent );
        }

        /**
         * Hilfsmethode toArray.
         * @param pCollection
         * @return
         */
        public static TimeEventHolder[] toArray(final Collection pCollection)
        {
            return (TimeEventHolder[]) pCollection.toArray(new TimeEventHolder[ pCollection.size() ]);
        }
    }

    /**
     * Verpacken des bergebenen {@link TimeEvent}-Array in ein
     * {@link TimeEventHolder}-Array mit Erhaltung der Information
     * linker oder rechter Operand.
     *
     * @param pTimeEventArr einzupackendes Array mit TimeEvents
     * @param pIsLeftTimeEvent linker Operand 'true', rechter Operand 'false'
     * @return
     */
    private static TimeEventHolder[] setInHolder( //--
            final TimeEvent[] pTimeEventArr, //--
            final boolean pIsLeftTimeEvent){
        TimeEventHolder[] timeEventHolderArr = new TimeEventHolder[ pTimeEventArr.length ];
        for (int i = 0; i < pTimeEventArr.length; i++)
    {
            timeEventHolderArr[i] = new TimeEventHolder( pTimeEventArr[i] , pIsLeftTimeEvent );
        }
        return timeEventHolderArr;
    }

    /**
     * Auspacken der im bergebenen {@link TimeEventHolder}-Array
     * enthaltenen {@link TimeEvent}s.
     *
     * @param pTimeEventHolderArr
     * @return
     */
    private static TimeEvent[] getFromHolder( //--
            final TimeEventHolder[] pTimeEventHolderArr)
    {
        TimeEvent[] timeEventArr = new TimeEvent[ pTimeEventHolderArr.length ];
        for (int i = 0; i < pTimeEventHolderArr.length; i++)
        {
            timeEventArr[i] = pTimeEventHolderArr[i].timeEvent;
        }
        return timeEventArr;
    }



    /**
     * UND-Verknpfung zweier Zeit-Ereignis-Listen
     * (Timelines).
     *
     * @param pTimeEventArrLeft
     * @param pTimeEventArrRight
     * @return
     */
    private static TimeEvent[] and( //--
            final TimeEvent[] pTimeEventArrLeft, //--
            final TimeEvent[] pTimeEventArrRight)
    {
        final TimeEventHolder[] joinTimeEventHolderArr = joinAndSortToHolderArr(pTimeEventArrLeft, pTimeEventArrRight);

        //Verarbeiten
        ArrayList dstList = new ArrayList();
        //Merker Status links
        boolean leftSts = pTimeEventArrLeft[0].type.getPostStat();
        //Merker Status rechts
        boolean rightSts = pTimeEventArrRight[0].type.getPostStat();
        for (int i = 0; i < joinTimeEventHolderArr.length; i++)
        {
//System.out.println( "i: " + i );
            TimeEventHolder timeEventHolder = joinTimeEventHolderArr[i];
            TimeEvent timeEvent = timeEventHolder.timeEvent;
//System.out.println( "timeEvent[" + i + "]: " + timeEvent + " isLeftTimeEvent: " + timeEventHolder.isLeftTimeEvent );
            //Mitschreiben der Vernderung links oder rechts
            if ( timeEventHolder.isLeftTimeEvent )
            {
                leftSts = timeEvent.type.getPostStat();
            }
            else
            {
                rightSts = timeEvent.type.getPostStat();
            }
//System.out.println( " leftSts: " + leftSts + " rightSts: " + rightSts );


            if ( leftSts && rightSts //--
                    && ( timeEvent.type.getPostStat()
                            // timeEvent.type == TimeEventType.START_1 //--
                            // || timeEvent.type == TimeEventType.END_1 //--
                            // || timeEvent.type == TimeEventType.FROM_0_TO_1 //--
                            // || timeEvent.type == TimeEventType.MOMENT_1 //--
                            ) )
            {
                dstList.add( timeEvent ); // TODO zu einem Statement zusammenfassen, Condition erweitern
//System.out.println( "add " + timeEvent.type );
            }
            else if ( ! ( leftSts && rightSts ) //--
                    && ( ! timeEvent.type.getPostStat()
                            // timeEvent.type == TimeEventType.START_0 //--
                            // || timeEvent.type == TimeEventType.END_0 //--
                            // || timeEvent.type == TimeEventType.FROM_1_TO_0 //--
                            // || timeEvent.type == TimeEventType.MOMENT_0 //--
                            ) )
            {
                dstList.add( timeEvent );
//System.out.println( "add " + timeEvent.type );
            }

        }

//System.out.println( "dstList: " + dstList );

        return TimeEvent.toArray( dstList );
    }


    /**
     * ODER-Verknpfung zweier Zeit-Ereignis-Listen
     * (Timelines).
     *
     * @param pTimeEventArrLeft
     * @param pTimeEventArrRight
     * @return
     */
    private static TimeEvent[] or( //--
            final TimeEvent[] pTimeEventArrLeft, //--
            final TimeEvent[] pTimeEventArrRight)
    {
        final TimeEventHolder[] joinTimeEventHolderArr = joinAndSortToHolderArr(pTimeEventArrLeft, pTimeEventArrRight);

        //Verarbeiten
        ArrayList dstList = new ArrayList();
        //Merker Status links
        boolean leftSts = pTimeEventArrLeft[0].type.getPostStat();
        //Merker Status rechts
        boolean rightSts = pTimeEventArrRight[0].type.getPostStat();
        for (int i = 0; i < joinTimeEventHolderArr.length; i++)
        {
//System.out.println( "i: " + i );
            TimeEventHolder timeEventHolder = joinTimeEventHolderArr[i];
            TimeEvent timeEvent = timeEventHolder.timeEvent;
//System.out.println( "timeEvent[" + i + "]: " + timeEvent + " isLeftTimeEvent: " + timeEventHolder.isLeftTimeEvent );
            //Mitschreiben der Vernderung links oder rechts
            if ( timeEventHolder.isLeftTimeEvent )
            {
                leftSts = timeEvent.type.getPostStat();
            }
            else
            {
                rightSts = timeEvent.type.getPostStat();
            }
//System.out.println( " leftSts: " + leftSts + " rightSts: " + rightSts );


            if ( ( leftSts || rightSts ) //--
                    && ( timeEvent.type == TimeEventType.START_1 //--
                            || timeEvent.type == TimeEventType.END_1 //--
                            || timeEvent.type == TimeEventType.FROM_0_TO_1 //--
                            || timeEvent.type == TimeEventType.MOMENT_1 //--
                            ) )
            {
                dstList.add( timeEvent ); // TODO zu einem Statement zusammenfassen, Condition erweitern
//System.out.println( "add " + timeEvent.type );
            }
            else if ( ! ( leftSts || rightSts ) //--
                    && ( timeEvent.type == TimeEventType.START_0 //--
                            || timeEvent.type == TimeEventType.END_0 //--
                            || timeEvent.type == TimeEventType.FROM_1_TO_0 //--
                            || timeEvent.type == TimeEventType.MOMENT_0 //--
                            ) )
            {
                dstList.add( timeEvent );
//System.out.println( "add " + timeEvent.type );
            }

        }

//System.out.println( "dstList: " + dstList );

        return TimeEvent.toArray( dstList );
    }


    /**
     * ENTWEDER-ODER-Verknpfung zweier
     * Zeit-Ereignis-Listen (Timelines).
     *
     * @param pTimeEventArrLeft
     * @param pTimeEventArrRight
     * @return
     */
    private static TimeEvent[] xor( //--
            final TimeEvent[] pTimeEventArrLeft, //--
            final TimeEvent[] pTimeEventArrRight)
    {
        final TimeEventHolder[] joinTimeEventHolderArr = joinAndSortToHolderArr(pTimeEventArrLeft, pTimeEventArrRight);

        //Verarbeiten
        ArrayList dstList = new ArrayList();
        //Merker Status links
        boolean leftSts = pTimeEventArrLeft[0].type.getPostStat();
        //Merker Status rechts
        boolean rightSts = pTimeEventArrRight[0].type.getPostStat();
        for (int i = 0; i < joinTimeEventHolderArr.length; i++)
        {
//System.out.println( "i: " + i );
            TimeEventHolder timeEventHolder = joinTimeEventHolderArr[i];
            TimeEvent timeEvent = timeEventHolder.timeEvent;
//System.out.println( "timeEvent[" + i + "]: " + timeEvent + " isLeftTimeEvent: " + timeEventHolder.isLeftTimeEvent );
            //Mitschreiben der Vernderung links oder rechts
            if ( timeEventHolder.isLeftTimeEvent )
            {
                leftSts = timeEvent.type.getPostStat();
            }
            else
            {
                rightSts = timeEvent.type.getPostStat();
            }
//System.out.println( " leftSts: " + leftSts + " rightSts: " + rightSts );


            if ( ( leftSts ^ rightSts ) //--
                    && ( timeEvent.type == TimeEventType.START_1 //--
                            || timeEvent.type == TimeEventType.END_1 //--
                            || timeEvent.type == TimeEventType.FROM_0_TO_1 //--
                            || timeEvent.type == TimeEventType.MOMENT_1 //--
                            ) )
            {
                dstList.add( timeEvent ); // TODO zu einem Statement zusammenfassen, Condition erweitern
//System.out.println( "add " + timeEvent.type );
            }
            else if ( ! ( leftSts ^ rightSts ) //--
                    && ( timeEvent.type == TimeEventType.START_0 //--
                            || timeEvent.type == TimeEventType.END_0 //--
                            || timeEvent.type == TimeEventType.FROM_1_TO_0 //--
                            || timeEvent.type == TimeEventType.MOMENT_0 //--
                            ) )
            {
                dstList.add( timeEvent );
//System.out.println( "add " + timeEvent.type );
            }

        }

//System.out.println( "dstList: " + dstList );

        return TimeEvent.toArray( dstList );
    }


    /**
     * Zusammenfgen und Sortieren zweier {@link TimeEvent}-Arrays
     * fr die logischen Operationen <b>and</b>, <b>or</b> sowie <b>xor</b>.
     *
  	 * @param pTimeEventArrLeft
  	 * @param pTimeEventArrRight
  	 * @return
  	 */
  	private static TimeEventHolder[] joinAndSortToHolderArr(//--
              final TimeEvent[] pTimeEventArrLeft, //--
              final TimeEvent[] pTimeEventArrRight)
    {
  	    final TimeEventHolder[] leftTimeEventHolderArr = setInHolder( pTimeEventArrLeft , true );
        final TimeEventHolder[] rightTimeEventHolderArr = setInHolder( pTimeEventArrRight , false );
        final ArrayList srcList = new ArrayList();

        srcList.addAll( Arrays.asList( leftTimeEventHolderArr ) );
        srcList.addAll( Arrays.asList( rightTimeEventHolderArr ) );

        // Sortieren
        Collections.sort( srcList );
        final TimeEventHolder[] joinTimeEventHolderArr = TimeEventHolder.toArray( srcList );
  		  return joinTimeEventHolderArr;
  	}

    /**
     * Test der Zeitbereichs-Operationen.
     *
     * @param args
     */
  	public static void main(String[] args)
    {
        {
            final TimeEvent[] timeEventArrLeft = new TimeEvent[]
            {
                    new TimeEvent( 0, TimeEventType.START_0),
                    new TimeEvent( 9, TimeEventType.FROM_0_TO_1),
                    new TimeEvent(29, TimeEventType.END_1)
            };

            System.out.println( buildOszillogramm( timeEventArrLeft ) );

            final TimeEvent[] timeEventArrRight = new TimeEvent[]
            {
                    new TimeEvent( 0, TimeEventType.START_1),
                    new TimeEvent(19, TimeEventType.FROM_1_TO_0),
                    new TimeEvent(29, TimeEventType.END_0)
            };

            System.out.println( buildOszillogramm( timeEventArrRight ) );

            System.out.println("  ============ AND ============");


            final TimeEvent[] timeEventArrResult = and( timeEventArrLeft , timeEventArrRight );
            //TODO Result-Test-Anweisungen
            System.out.println( buildOszillogramm( timeEventArrResult ) );
        }
        System.out.println("****************************************************************************");
        {
            final TimeEvent[] timeEventArrLeft = new TimeEvent[]
            {
                    new TimeEvent( 0, TimeEventType.START_1),
                    new TimeEvent( 9, TimeEventType.FROM_1_TO_0),
                    new TimeEvent(29, TimeEventType.END_0)
            };

            System.out.println( buildOszillogramm( timeEventArrLeft ) );

            final TimeEvent[] timeEventArrRight = new TimeEvent[]
            {
                    new TimeEvent( 0, TimeEventType.START_0),
                    new TimeEvent(19, TimeEventType.FROM_0_TO_1),
                    new TimeEvent(29, TimeEventType.END_1)
            };

            System.out.println( buildOszillogramm( timeEventArrRight ) );

            System.out.println("  ============ OR =============");

            final TimeEvent[] timeEventArrResult = or( timeEventArrLeft , timeEventArrRight );
            //TODO Result-Test-Anweisungen
            System.out.println( buildOszillogramm( timeEventArrResult ) );

        }
        System.out.println("****************************************************************************");
        {
            final TimeEvent[] timeEventArrLeft = new TimeEvent[]
            {
                    new TimeEvent( 0, TimeEventType.START_1),
                    new TimeEvent( 9, TimeEventType.FROM_1_TO_0),
                    new TimeEvent(29, TimeEventType.END_0)
            };

            System.out.println( buildOszillogramm( timeEventArrLeft ) );

            final TimeEvent[] timeEventArrRight = new TimeEvent[]
            {
                    new TimeEvent( 0, TimeEventType.START_0),
                    new TimeEvent(19, TimeEventType.FROM_0_TO_1),
                    new TimeEvent(29, TimeEventType.END_1)
            };

            System.out.println( buildOszillogramm( timeEventArrRight ) );

            System.out.println("  ============ XOR ============");

            final TimeEvent[] timeEventArrResult = xor( timeEventArrLeft , timeEventArrRight );
            //TODO Result-Test-Anweisungen
            System.out.println( buildOszillogramm( timeEventArrResult ) );

        }
        //TODO NOT-Verknpfung
    }

}
